# $File: //depot/OurNet-BBS/BBS/MAPLE2/GroupGroup.pm $ $Author: autrijus $
# $Revision: #5 $ $Change: 1525 $ $DateTime: 2001/08/17 22:49:33 $

package OurNet::BBS::MAPLE2::GroupGroup;

use strict;
use fields qw/_ego _hash/;
use OurNet::BBS::Base;

sub refresh_meta {
    die 'Group for MAPLE2 not implemented.';
}

1;
