# $File: //depot/OurNet-BBS/BBS/MELIX/BoardGroup.pm $ $Author: autrijus $
# $Revision: #4 $ $Change: 1525 $ $DateTime: 2001/08/17 22:49:33 $

package OurNet::BBS::MELIX::BoardGroup;

use strict;
use base qw/OurNet::BBS::MAPLE3::BoardGroup/;
use fields qw/_ego _hash/;
use OurNet::BBS::Base;

1;
